﻿using System;
using System.Collections.Generic;
using System.IO;
using System.IO.Compression;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Infragistics.Persistence;
using Infragistics.Persistence.Primitives;

namespace XamGridPersistence
{
    public static class ConfigHelper
    {


        private static PersistenceSettings _settings;
        private static PersistenceSettings Settings
        {
            get
            {
                if (_settings == null)
                {
                    _settings = new PersistenceSettings
                    {
                        SavePersistenceOptions = PersistenceOption.OnlySpecified,
                        LoadPersistenceOptions = PersistenceOption.OnlySpecified
                    };
                    _settings.PropertySettings.Add(new PropertyNamePersistenceInfo
                    {
                        PropertyName = "Columns",
                        Options = PropertyNamePersistenceOptions.PropertyName
                    });
                    _settings.PropertySettings.Add(new PropertyNamePersistenceInfo
                    {
                        PropertyName = "GroupByColumns",
                        Options = PropertyNamePersistenceOptions.PropertyName
                    });
                    _settings.PropertySettings.Add(new PropertyNamePersistenceInfo
                    {
                        PropertyName = "SortedColumns",
                        Options = PropertyNamePersistenceOptions.PropertyName
                    });
                }


                return _settings;
            }
        }

        public static byte[] SerializeConfiguration(BasisGrid grid)
        {

            MemoryStream mStream = null;
            MemoryStream outStream = null;
            byte[] result;
            try
            {
                mStream = PersistenceManager.Save(grid, Settings);
                outStream = new MemoryStream();
                using (var tinyStream = new GZipStream(outStream, CompressionMode.Compress))
                    mStream.CopyTo(tinyStream);
                result = outStream.ToArray();
            }
            finally
            {
                if (outStream != null)
                    outStream.Dispose();

                if (mStream != null)
                    mStream.Dispose();
            }
            return result;
        }

        public static Boolean DeserializeConfiguration(Config config, BasisGrid grid)
        {
            MemoryStream inputStream = null;
            MemoryStream bigStreamOut = null;
            try
            {
                inputStream = new MemoryStream(config.Value);
                bigStreamOut = new MemoryStream();
                using (var bigStream = new GZipStream(inputStream, CompressionMode.Decompress))
                    bigStream.CopyTo(bigStreamOut);
                bigStreamOut.Position = 0;
                PersistenceManager.Load(grid, bigStreamOut);

                return true;
            }
            catch
            {
                return false;
            }
            finally
            {
                if (inputStream != null)
                    inputStream.Dispose();

                if (bigStreamOut != null)
                    bigStreamOut.Dispose();
            }
        }

    }
}
